import re

from extras.plugins import PluginTemplateExtension  # pylint: disable=import-error

from . import livestatus


class NagiosStatus(PluginTemplateExtension):
    def __init__(self, context):
        super().__init__(context)
        self.settings = self.context["settings"].PLUGINS_CONFIG["netbox_nagios"]
        self.hostname = self.context["object"].name or ""  # name can be None.
        self.livestatus_host = self.get_livestatus_host()
        self.livestatus_port = self.settings["livestatus_port"]
        self.nagios_base_url = self.get_nagios_base_url()

    def get_livestatus_host(self):
        """Uses settings and potential overrides to determine the Nagios host."""
        for regex, livestatus_host in self.settings["livestatus_host_overrides"]:
            if re.search(regex, self.hostname):
                return livestatus_host
        return self.settings["livestatus_host"]

    def get_nagios_base_url(self):
        """Uses settings and potential overrides to determine the Nagios url."""
        for regex, nagios_base_url in self.settings["nagios_base_url_overrides"]:
            if re.search(regex, self.hostname):
                return nagios_base_url
        return self.settings["nagios_base_url"]

    def buttons(self):
        """Adds an extra button at the top of the bage."""
        return self.render(
            "device_nagios_buttons.html",
            extra_context={"nagios_base_url": self.nagios_base_url},
        )

    def right_page(self):
        """Adds a status table to the page."""
        extra_context = {
            "nagios_base_url": self.nagios_base_url,
        }
        try:
            extra_context["nagios"] = livestatus.hoststatus(
                self.hostname, self.livestatus_host, self.livestatus_port,
            )
        except Exception:  # pylint: disable=broad-except
            # Be very defensive so that broken Nagios doesn't break Netbox.
            pass
        return self.render("device_nagios_box.html", extra_context=extra_context)


class NagiosStatusDevice(NagiosStatus):
    model = "dcim.device"


class NagiosStatusVM(NagiosStatus):
    model = "virtualization.virtualmachine"


template_extensions = [  # pylint: disable=invalid-name
    NagiosStatusDevice,
    NagiosStatusVM,
]



from django import template  # pylint: disable=import-error

register = template.Library()  # pylint: disable=invalid-name


@register.filter(is_safe=True)
def nagios_status_color(value: int):
    local_colors = {0: "success", 1: "warning", 2: "danger", 3: "info"}
    return local_colors.get(value, "default")


@register.filter(is_safe=True)
def nagios_status_string(value: int):
    strings = {0: "OK", 1: "WARN", 2: "CRIT", 3: "UNKN"}
    return strings.get(value, "NO DATA")



import json
import socket

TIMEOUT = 3  # seconds
BUFFER_SIZE = 4096  # bytes


def hoststatus(hostname: str, livestatus_host: str, livestatus_port: int):
    """Fetches livestatus from Nagios about hostname."""
    query = (
        "GET hosts\n"
        + "Filter: host_name = %s\n" % hostname
        + "OutputFormat: json\n"
        + "\n"
    )

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(TIMEOUT)
    s.connect((livestatus_host, livestatus_port))
    s.sendall(query.encode("utf-8"))
    s.shutdown(socket.SHUT_WR)

    data = []
    while True:
        buf_data = s.recv(BUFFER_SIZE)
        if not buf_data:
            break
        data.append(buf_data)
    s.close()

    if not data:
        return None
    data = json.loads(b"".join(data))

    if not data or len(data) <= 1:
        return None
    return dict(zip(data[0], data[1]))



try:
    from extras.plugins import PluginConfig
except ImportError:
    # Dummy so install of wheel works without Netbox.
    class PluginConfig(object):
        pass


from .version import VERSION


class NetboxNagiosConfig(PluginConfig):
    """
    This class defines attributes for the NetBox Nagios plugin.
    """

    # Plugin package name
    name = "netbox_nagios"

    # Human-friendly name and description
    verbose_name = "Nagios"
    description = "Plugin to show Nagios status in Netbox"

    # Plugin version
    version = VERSION

    # Plugin author
    author = "Hudson River Trading LLC"
    author_email = "opensource@hudson-trading.com"

    # Configuration parameters that MUST be defined by the user (if any)
    required_settings = ["livestatus_host", "nagios_base_url"]

    # Default configuration parameter values, if not set by the user
    default_settings = {
        "livestatus_host_overrides": [],
        "livestatus_port": 6557,
        "nagios_base_url_overrides": [],
    }

    # Base URL path. If not set, the plugin name will be used.
    base_url = "nagios"

    # Caching config
    caching_config = {}


config = NetboxNagiosConfig  # pylint: disable=invalid-name



from setuptools import find_packages, setup

from netbox_nagios.version import VERSION

setup(
    name="netbox-nagios",
    version=VERSION,
    author="Hudson River Trading LLC",
    author_email="opensource@hudson-trading.com",
    description="Netbox Plugin to show Nagios device state in Netbox.",
    url="https://github.com/hudson-trading/netbox_nagios/",
    license="New BSD",
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
)
